
package com.marce.test;

/**
 * Test de la sentencia switch en las diferentes versiones de Java.
 *
 * @author Marce
 */

public class TestSwitchColor {

    public enum Color {
        ROJO, NARANJA, AMARILLO, CHARTREUSE, LIMA, PRIMAVERAL, AGUA, AZUR, AZUL, VIOLETA, FUCSIA, ROSA
    }

    /**
     * Formato de color en hexadecimal para css
     *
     * @param id color
     * @return color hexadecimal para css
     */
    public static String getHexColor(int nColor) {
        Color eColor = Color.values()[nColor];
        int hexColor;
        // hexColor = getColorJava4(nColor);
        // hexColor = getColorJava5(eColor);
        // hexColor = getColorJava7(eColor.name());
        hexColor = getColorJava12(eColor);
        String cHexColor = String.format("%S%06X", "#", hexColor);
        return cHexColor;
    }

    /**
     * Determina color en hexadecimal (Java hasta 1.4)
     *
     * @param id color
     * @return color hexadecimal
     */
    private static int getColorJava4(int nColor) {
        int hexColor;

        // Java hasta 1.4 se permitía utilizar la sentencia switch con los tipos de
        // datos primitivos byte, short, char, int.

        switch (nColor) {
        case 0: // ROJO
            hexColor = 0xFF0000;
            break;
        case 1: // NARANJA
            hexColor = 0xFF8000;
            break;
        case 2: // AMARILLO
            hexColor = 0xFFFF00;
            break;
        case 3: // CHARTREUSE
            hexColor = 0x80FF00;
            break;
        case 4: // LIMA
            hexColor = 0x00FF00;
            break;
        case 5: // PRIMAVERAL
            hexColor = 0x00FF80;
            break;
        case 6: // AGUA
            hexColor = 0x00FFFF;
            break;
        case 7: // AZUR
            hexColor = 0x0080FF;
            break;
        case 8: // AZUL
            hexColor = 0x0000FF;
            break;
        case 9: // VIOLETA
            hexColor = 0x8000FF;
            break;
        case 10: // FUCSIA
            hexColor = 0xFF00FF;
            break;
        case 11: // ROSA
            hexColor = 0xFF0080;
            break;
        default:
            throw new IllegalArgumentException("Java hasta 1.4 | Valor desconocido: " + nColor);
        }

        return hexColor;
    }

    /**
     * Determina color en hexadecimal (Java 5)
     *
     * @param id color
     * @return color hexadecimal
     */
    private static int getColorJava5(Color eColor) {
        int hexColor;

        // Desde Java 5 también se admiten los tipos enumerados y las clases de
        // envoltura Byte, Short, Character, Integer.

        switch (eColor) {
        case ROJO:
            hexColor = 0xFF0000;
            break;
        case NARANJA:
            hexColor = 0xFF8000;
            break;
        case AMARILLO:
            hexColor = 0xFFFF00;
            break;
        case CHARTREUSE:
            hexColor = 0x80FF00;
            break;
        case LIMA:
            hexColor = 0x00FF00;
            break;
        case PRIMAVERAL:
            hexColor = 0x00FF80;
            break;
        case AGUA:
            hexColor = 0x00FFFF;
            break;
        case AZUR:
            hexColor = 0x0080FF;
            break;
        case AZUL:
            hexColor = 0x0000FF;
            break;
        case VIOLETA:
            hexColor = 0x8000FF;
            break;
        case FUCSIA:
            hexColor = 0xFF00FF;
            break;
        case ROSA:
            hexColor = 0xFF0080;
            break;
        default:
            throw new IllegalArgumentException("Java 5 | Valor desconocido: " + eColor);
        }
        return hexColor;
    }

    /**
     * Determina color en hexadecimal (Java 7)
     *
     * @param id color
     * @return color hexadecimal
     */
    private static int getColorJava7(String cColor) {
        int hexColor;

        // En Java 7 se introdujo soporte para usar switch con String

        switch (cColor) {
        case "ROJO":
            hexColor = 0xFF0000;
            break;
        case "NARANJA":
            hexColor = 0xFF8000;
            break;
        case "AMARILLO":
            hexColor = 0xFFFF00;
            break;
        case "CHARTREUSE":
            hexColor = 0x80FF00;
            break;
        case "LIMA":
            hexColor = 0x00FF00;
            break;
        case "PRIMAVERAL":
            hexColor = 0x00FF80;
            break;
        case "AGUA":
            hexColor = 0x00FFFF;
            break;
        case "AZUR":
            hexColor = 0x0080FF;
            break;
        case "AZUL":
            hexColor = 0x0000FF;
            break;
        case "VIOLETA":
            hexColor = 0x8000FF;
            break;
        case "FUCSIA":
            hexColor = 0xFF00FF;
            break;
        case "ROSA":
            hexColor = 0xFF0080;
            break;
        default:
            throw new IllegalArgumentException("Java 7 | Valor desconocido: " + cColor);
        }
        return hexColor;
    }

    /**
     * Determina color en hexadecimal (Java 12)
     *
     * @param id color
     * @return color hexadecimal
     */
    @SuppressWarnings("preview")
    private static int getColorJava12(Color eColor) {

        // Java 12 extiende la sentencia switch para que pueda utilizarse como una
        // declaración o una expresión, y que de ambas maneras pueda usarse en el
        // formato tradicional (con break) [... :] o simplificado (sin break) [... ->].

        // Ahora lo que queremos es posible hacerlo directamente en una sentencia switch
        // de la siguiente forma.

        return switch (eColor) {
        case ROJO -> 0xFF0000;
        case NARANJA -> 0xFF8000;
        case AMARILLO -> 0xFFFF00;
        case CHARTREUSE -> 0x80FF00;
        case LIMA -> 0x00FF00;
        case PRIMAVERAL -> 0x00FF80;
        case AGUA -> 0x00FFFF;
        case AZUR -> 0x0080FF;
        case AZUL -> 0x0000FF;
        case VIOLETA -> 0x8000FF;
        case FUCSIA -> 0xFF00FF;
        case ROSA -> 0xFF0080;
        default -> throw new IllegalArgumentException("Java 12 | Valor desconocido: " + eColor);
        };

    }

}
